from ._utils import *

from ..models import RelatedContent
from ..models import RelatedType


class RelatedContentManagerTestCase(TestCase):
    def test_by_type_returns_an_empty_queryset_if_no_match_is_found(self):
        one, two = generate_fake_articles(2)

        self.assertEqual([], list(one.related.by_type("unknown")))

    def test_by_type_returns_matching_related_values_by_type_title(self):
        one, two = generate_fake_articles(2)

        t = RelatedType.objects.create(title="articles")
        c = RelatedContent.objects.create(
            related_type=t,
            source_object=one,
            destination_object=two
        )

        destination_objects = [a.destination_object for a in
                one.related.by_type("articles")]
        self.assertTrue(two in destination_objects)



from contextlib import contextmanager
import copy
from django import template
from django.contrib.contenttypes.models import ContentType
from django.template import RequestContext
from django.template.base import TemplateSyntaxError, VariableDoesNotExist
from ..models import RelatedContent

register = template.Library()


class LeadArtNode(template.Node):
    def __init__(self, from_object, preset):
        self.from_object = template.Variable(from_object)
        self.preset = template.Variable(preset)

    def render(self, context):
        obj = self.from_object.resolve(context)
        obj_type = ContentType.objects.get_for_model(obj)
        related_art = RelatedContent.objects.by_type('lead_art')\
                                            .filter(source_type=obj_type)\
                                            .filter(source_id=obj.id)
        try:
            art = related_art[0].destination_object
        except IndexError:
            return ''
        return art.render_visual(self.preset.resolve(context))


@register.tag
def lead_art(parser, token):
    tokens = token.split_contents()
    if len(tokens) is 3:
        _, from_object, preset = tokens
        return LeadArtNode(from_object, preset)

    message = "Too %s parameters" % ("many" if len(tokens) > 3 else "few")
    raise TemplateSyntaxError(message)



from django.contrib.contenttypes.generic import GenericRelation

from .models import RelatedContent


class RelatedContentField(GenericRelation):
    def __init__(self, **kwargs):
        defaults = {
            "object_id_field": "source_id",
            "content_type_field": "source_type",
        }
        defaults.update(kwargs)
        super(RelatedContentField, self).__init__(RelatedContent, **defaults)
        
from south.modelsinspector import add_ignored_fields
add_ignored_fields(["^armstrong\.apps\.related_content\.fields\.RelatedContentField"])


from django.db.models import ForeignKey
import fudge
import random
from ._utils import TestCase

from .. import admin
from ..models import RelatedContent
from ..admin import formfield_for_foreignkey_helper


def generate_fake_settings():
    settings = fudge.Fake()
    settings.has_attr(**{
        "ARMSTRONG_RELATED_TYPE_INITIAL_FILTER": {
            "title": "articles",
        }
    })
    return settings


def generate_fake_qs():
    qs = fudge.Fake()
    qs.provides("get")
    return qs


def generate_fake_type(qs=None):
    if not qs:
        qs = generate_fake_qs()
    type = fudge.Fake()
    type.has_attr(objects=qs)
    return type


def generate_fakes():
    fake_settings = generate_fake_settings()
    fake_qs = generate_fake_qs()
    fake_type = generate_fake_type(fake_qs)

    return {
        "settings": fake_settings,
        "qs": fake_qs,
        "type": fake_type,
    }


def random_args():
    return [str(random.randint(1000, 2000))
            for i in range(random.randint(1, 5))]


def random_kwargs():
    return dict(zip(random_args(), random_args()))


def random_args_and_kwargs():
    return random_args(), random_kwargs()


class RelatedContentInlineTestCase(TestCase):
    def setUp(self):
        self.model = RelatedContent()
        self.db_field = self.get_db_field_by_name("related_type")

    def get_db_field_by_name(self, name):
        return self.model._meta.get_field_by_name(name)[0]

    def test_returns_args_untouched_if_not_related_type(self):
        args, kwargs = random_args_and_kwargs()
        content_type_field = self.get_db_field_by_name("source_type")
        ret = formfield_for_foreignkey_helper({}, content_type_field,
                *args, **kwargs)
        returned_args = list(ret[0])
        db_field = returned_args.pop(0)
        # Test db_field separately to avoid false failure in Django
        self.assertIsA(db_field, ForeignKey)
        self.assertEqual(args, returned_args)

    def test_returns_kwargs_untouched_if_not_related_type(self):
        args, kwargs = random_args_and_kwargs()
        content_type_field = self.get_db_field_by_name("source_type")
        ret = formfield_for_foreignkey_helper({}, content_type_field,
                *args, **kwargs)
        returned_kwargs = ret[1]
        self.assertEqual(kwargs, returned_kwargs)

    def test_adds_initial_if_it_is_not_present(self):
        random_return = random.randint(10000, 20000)
        args, kwargs = random_args_and_kwargs()
        fake_settings = generate_fake_settings()
        fake_qs = fudge.Fake()
        fake_qs.expects("get").with_args(title="articles").returns(random_return)
        fake_type = generate_fake_type(fake_qs)
        with fudge.patched_context(admin, "RelatedType", fake_type):
            with fudge.patched_context(admin, "settings", fake_settings):
                ret = formfield_for_foreignkey_helper({}, self.db_field,
                        *args, **kwargs)
                returned_kwargs = ret[1]
        self.assertTrue("initial" in returned_kwargs)
        self.assertEqual(returned_kwargs["initial"], random_return)

    def test_ignores_initial_if_provided(self):
        expected_return = random.randint(30000, 40000)
        random_return = random.randint(10000, 20000)
        args, kwargs = random_args_and_kwargs()
        kwargs["initial"] = expected_return
        fake_settings = generate_fake_settings()
        fake_qs = fudge.Fake()
        fake_qs.expects("get").with_args(title="articles").returns(random_return)
        fake_type = generate_fake_type(fake_qs)
        with fudge.patched_context(admin, "RelatedType", fake_type):
            with fudge.patched_context(admin, "settings", fake_settings):
                ret = formfield_for_foreignkey_helper({}, self.db_field,
                        *args, **kwargs)
                returned_kwargs = ret[1]
        self.assertTrue("initial" in returned_kwargs)
        self.assertEqual(returned_kwargs["initial"], expected_return)
